// Bouton.h: interface for the Bouton class.
//
//////////////////////////////////////////////////////////////////////

#ifndef BOUTON_H
#define BOUTON_H

#include "RULES_Texture.h"

/**
 * classe gerant un bouton basique de l'interface graphique
 *
 * un bouton sert a savoir quand l'utilisateur veut effectuer une commande (lancer la partie, changer de menu...).
 * Le bouton a 2 images :
 * - une image normal quand il est au "repos"
 * - une image quand la souris passe dessus.
 * Pour savoir si la souris est dessus un bouton, il faut utiliser la methode Bouton::isIn(int mouseX, int mouseY).
 * Le bouton est capable de se dessiner tout seul grace a la methode draw()
 */
class Bouton  
{
private:
	/**
	 * La largeur du bouton (en pixels)
	 */
	int width;
	
	/**
	 * la hauteur du bouton (en pixels)
	 */
	int height;

	/**
	 * la position Y du bouton (en pixels)
	 */
	int y;

	/**
	 * la position X du bouton (en pixels)
	 */
	int x;

	/**
	 * la texture utilise pour afficher le bouton quand le souris est dessus
	 */
	RULES_Texture * imagePassed;

	/**
	 * la texture utilise pour afficher le bouton en mode normal
	 */
	RULES_Texture * imageNormal;
public:
	/**
	 * methode permettant de definire la largeur du bouton
	 * @param w: la largeur (en pixels)
	 */
	void setWidth(int w);

	/**
	 * methode permettant de definire la hauteur du bouton
	 * @param h: la hauteur du bouton (en pixels)
	 */
	void setHeight(int h);

	/**
	 * methode permettant de definire la position du bouton
	 * @param posX: la position X du bouton en pixels
	 * @param posY: la position Y du bouton en pixels
	 */
	void setPosition(int posX, int posY);

	/**
	 * methode permettant de definire la texture utilise lorsque la souris passe dessus le bouton.
	 * <B>/!\ATTENTION/!\</B> La texture est possede par le bouton et est donc suprime par le destructeur
	 * du bouton. Ne pas esayer de la deleter de l'exterieur
	 * @param text: La texture
	 */
	void setImagePassed(RULES_Texture * text);

	/**
	 * methode permettant de definire la texture utilise lorsque d'un affichage normal
	 * <B>/!\ATTENTION/!\</B> La texture est possede par le bouton et est donc suprime par le destructeur
	 * du bouton. Ne pas esayer de la deleter de l'exterieur
	 * @param text: La texture
	 */
	void setImageNormal(RULES_Texture * text);

	/**
	 * methode dessinant le bouton a l'ecran.
	 * @param mouseX: la position X de la souris (en pixels)
	 * @param mouseY: la position Y de la souris (en pixels)
	 */
	void draw(int mouseX, int mouseY);

	/**
	 * definit la position gauche du bouton
	 * @param left: la position gauche du bouton (en pixels)
	 */
	void setLeft(int left);

	/**
	 * definit la position du haut du bouton
	 * @param top: le haut du bouton (en pixels)
	 */
	void setTop(int top);

	/**
	 * retourne la position gauche du bouton
	 * @return la position gauche du bouton
	 */
	int getLeft();

	/**
	 * retourne le haut du bouton
	 * @return le haut du bouton
	 */
	int getTop();

	/**
	 * permet de savoir si la souris est plac sur le bouton
	 * @param mouseX: la position X de la souris (en pixels)
	 * @param mouseY: la position Y de la souris (en pixels)
	 */
	bool isIn(int mouseX, int mouseY);

	/**
	 * constructeur par defaut
	 */
	Bouton();

	/**
	 * destructeur par defaut.
	 * Le destructeur detruit les textures utilise pour l'affichage.
	 */
	virtual ~Bouton();

};

#endif
