// Font.h: interface for the Font class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FONT_H__1F36C2FA_7629_4321_AA97_46D6C9383579__INCLUDED_)
#define AFX_FONT_H__1F36C2FA_7629_4321_AA97_46D6C9383579__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <gl\glaux.h>
#include <gl\glext.h>
#include <stdio.h>

/**
*	Classe permettant d'afficher des chaine de caractres  l'cran en Utilisant OpenGL
*et une texture de font.
*
* REM : il faut appel tout les mthode de rglage avant d'appeler la mthode 'buildFont'.
*/
class Font  
{
private:

	/**
	* la hauteur en pixel d'un caractere
	*/
	int charHeight;

	/**
	* la largeur en pixel d'un caractere
	*/
	int charWidth;

	/**
	* la largeur en pixel d'un espace entre chaque caractere
	*/
	int espace;

	/**
	* le nombre de caractere de la texture en largeur
	*/
	int xCharNumber;

	/**
	* le nombre de caractere de la texture en hauteur
	*/
	int yCharNumber;

	/**
	*	numero de la premiere liste qui est un caractere (pour enlever les
	* caractere speciaux comme ECHAP et autre (par default dans la table ascii c'est 32))
	*/
	int premierChar;

	/**
	* la hauteur de l'ecran
	*/
	int screenHeight;

	/**
	* la largeur de l'ecran
	*/
	int screenWidth;

	/**
	* l'ID de la texture de font
	*/
	unsigned int texture_ID;

	/**
	* la 1ere display list qui contiendra les caracteres
	*/
	unsigned int font;
public:

	/**
	* Constructeur
	*/
	Font();

	/**
	* Destructeur
	*/
	virtual ~Font();

	/**
	*	Il faut absolument appel cette mthode pour pouvoir ensuite rendre les chaines de caractres.
	*Cette mthode prend en compte les paramtre xCharNumber, xCharNumber et texture_ID pass
	*dans la mthode 'setTextureProperties'.
	*/
	void buildFont();

	/**
	* Permet d'afficher une chaine de caractres  l'cran
	*
	* @param x la position x sur l'cran
	* @param y la position y sur l'cran
	* @param chaine la chaine d'affichage genre printf(). exemple "il y a %d textures".
	* @param ... la suitede paramtre  intgrer  la chaine afficher
	*/
	void glText2d(int x, int y, const char *chaine, ...);

	/**
	* Regle les paramtre de l'cran
	*
	* @param width la largeur de l'cran
	* @param height la hauteur de l'cran
	*/
	void setScreenSize(int width,int height);

	/**
	* Regle les paramtres d'affichage des caractres
	*
	* @param width la largeur d'un caractre
	* @param height la hauteur d'un caractre
	* @param space l'espace entre deux caractre
	*/
	void setCaractereSize(int width,int height,int space);

	/**
	* Regle les paramtre de la texture OpenGL
	*
	* @param texture_ID l'identifiant OpenGL de la texture a afficher
	* @param xNumber le nombre de caractre dans la texture en largeur
	* @param yNumber le nombre decaraactre dans la texture en hauteur
	*/
	void setTextureProperties(unsigned int texture_ID,int xNumber,int yNumber);

	/**
	* Permet d'enlever les caractre spciaux dans la texture.
	*
	* @param base numero de la premiere liste qui est un caractere (pour enlever les caractere speciaux comme ECHAP et autre (par default dans la table ascii c'est 32))
	*/
	void setBaseCaractere(int base);

};

#endif // !defined(AFX_FONT_H__1F36C2FA_7629_4321_AA97_46D6C9383579__INCLUDED_)
