// Menu.h: interface for the Menu class.
//
//////////////////////////////////////////////////////////////////////

#ifndef GLUT_MENU
#define GLUT_MENU

#ifdef WIN32
#include <windows.h>
#endif

#include <GL/GL.h>
#include "glut.h"
#include <vector>
#include "RULES_Texture.h"


/**
 * la classe de menu du jeu.
 *
 * cette classe permet d'implementer des menu basics.
 * chaque type de menu herite de cette classe en redeffinissant les methodes principales comme le dessin et autre.
 */
class Menu : public GLUT
{
protected:
	/**
	 * l'image de fond du menu
	 */
	RULES_Texture * image;
public:
	/**
	 * constructeur par defaut
	 */
	Menu();

	/**
	 * destructeur virtuel pure
	 */
	virtual ~Menu(){};

	/**
	 * methode de dessin virtuelle pure
	 * voir GLUT::draw(void)
	 */
	virtual void draw(void) = 0;

	/**
	 * methode d' attente virtuelle pure
	 * voir GLUT::idle(void)
	 */
	virtual void idle(void) = 0;

	/**
	 * methode de clic souris virtuelle pure
	 * voir GLUT::mouseClicked(int boutton,int etat,int x,int y)
	 */
	virtual void mouseClicked(int boutton,int etat,int x,int y) = 0;

	/**
	 * methode de mouvement de la souris virtuelle pure
	 * voir GLUT::mouseMoved(int x,int y)
	 */
	virtual void mouseMoved(int x,int y) = 0;

	/**
	 * methode de mouvement de la souris avec clic virtuelle pure
	 * voir GLUT::mouseMovedClicked(int x,int y)
	 */
	virtual void mouseMovedClicked(int x,int y) = 0;

	/**
	 * methode de d'appuie sur une touche virtuelle pure
	 * voir GLUT::keyPressed(unsigned char key,int x,int y)
	 */
	virtual void keyPressed(unsigned char key,int x,int y) = 0;

	/**
	 * methode de relache d'une touche virtuelle pure
	 * voir GLUT::keyReleased(unsigned char key,int x,int y)
	 */
	virtual void keyReleased(unsigned char key,int x,int y) = 0;

	/**
	 * methode virtuelle pure utilise lorsqu'on presse une touche special (shift, fleches)
	 * @param key: le code (unicode) de la touche appuy
	 * @param x: la position X de la souris a l'ecran (en pixels)
	 * @param y: pa position Y de la souris a l'ecran (en pixels)
	 */
	virtual void specialKeyFunc(int key,int x,int y) = 0;

	/**
	 * methode virtuelle pure utilise lorsqu'on relache une touche special (shift, fleches)
	 * @param key: le code (unicode) de la touche appuy
	 * @param x: la position X de la souris a l'ecran (en pixels)
	 * @param y: pa position Y de la souris a l'ecran (en pixels)
	 */
	virtual void specialKeyUpFunc(int key,int x,int y) = 0;

	/**
	 * methode de redimentionnement de la fenetre virtuelle pure
	 * voir GLUT::reshape(int largeur,int hauteur)
	 */
	virtual void reshape(int largeur,int hauteur) = 0;

};

#endif
