// Model.h: interface for the Model class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(MODEL_H)
#define MODEL_H


#include "RULES_Vector.h"// pour avoir les vecteurs...
#include <stdio.h>
#include "OBJ_Reader.h"
#include <vector>
#include <math.h>
#include "RULES_Texture.h"	// Added by ClassView
#include "BoundingBox.h"	// Added by ClassView

/**
* La structure regroupant les information d'un vertex
*/
struct ModelVertex
{
	/**
	* Le vertex
	*/
	RULES_Vector vertex;
	/**
	* Les coordonnes de texture 2D
	*/
	float texCoord[2];
	/**
	* La normal
	*/
	RULES_Vector normal;
};

/**
* La classe model contenant tout les information pour rendre un model  l'cran.
*
* Comment par Hillaire Sbastien
*/
class Model  
{
private :
	/**
	* La bouding box du model.
	* Sert pour le frustum culling
	*/
	BoundingBox boundingBox;

	/**
	* La texture associe au model
	*/
	RULES_Texture * texture;

	/**
	* Le rayon du cylindre de l'objet
	* Sert pour ls collsions
	*/
	float radius;
	
	/**
	 * la liste des vertex
	 */
	ModelVertex * vertex;

	/**
	 * la liste des triangles (ce sont en fait les indices dans le tableau de vertex)
	 * un triangle est un ensemble de 3 ModelVertex
	 * la taille du tableau point par le triangle doit OBLIGATOIREMENT etre un multiple de 3
	 */
	int * triangles;

	/**
	 * le nombre de triangles
	 */
	unsigned int vertexNumber;

	/**
	* Est appel pour rgler les attribut de collision et de culling du model lors du chargement
	*/
	void transforme();

public:

	/**
	* Accder  la boundingbox du model
	*
	* @return la boundingbox du model
	*/
	BoundingBox * getBoundingBox();

	/**
	* Accder  la texture du model
	*
	* @return la texture du model
	*/
	RULES_Texture * getTexture();

	/**
	* Affecte la texture du model
	* REM : seul le pointeur est copi
	*
	* @param tex la texture utilis pour ce model
	*/
	void setTexture(RULES_Texture * tex);

	/**
	* Accder au rayon d'action du model
	*
	* @return le rayon d'action du model
	*/
	float getRadius();

	/**
	* Accder au nombre de vertex du model
	*
	* @return le nombre de vertex du model
	*/
	unsigned int getVertexNumber();

	/**
	* Accder au triangles du model
	*
	* @return la liste d'indices des vertices  trac pour afficher le model
	*/
	int * getTriangles();

	/**
	* Accder au vertices du model
	*
	* @return la liste des vertices du model
	*/
	ModelVertex * getVertex();

	/**
	* Charge le model  partir d'un fichier .obj
	*
	* @param file le chemin du model  charger
	*/
	void load(char * file);

	/**
	* Constructeur
	*/
	Model();

	/**
	* Destructeur
	*/
	virtual ~Model();

};

#endif 