// OBJ_Reader.cpp: implementation of the OBJ_Reader class.
//
//////////////////////////////////////////////////////////////////////

#include "OBJ_Reader.h"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

OBJ_Reader::OBJ_Reader(char *fileName)
{
	file=fileName;
}

OBJ_Reader::~OBJ_Reader()
{
	// on nettoie les vecteurs
	faceTab.clear();
	vertexTab.clear();
	normalTab.clear();
	textureTab.clear();

}


int OBJ_Reader::readFile()
{
	int ret = OBJ_READER_FILE_NOT_EXIST;// variable retourn
	char c;// caractere temporaire qui sera lu comme entete de chaque ligne
	char temp[255] = {0};//chaine temporaire servant  passer  la ligne.

	// on ouvre le fichier en lecture seul
	fichier = fopen(file,"r");
	// est ce que le fichier est bien ouvert ??
	if (fichier)
	{
		// tant qu'on est pas  la fin du fichier
		while (!feof(fichier))
		{
			// on lit le premier caractere de la ligne
			fscanf(fichier,"%c",&c);
			if (c == '\n')// test le cas ou la ligne serait vide
			{
				fscanf(fichier,"%c",&c);
			}
			// selon le caractere
			switch(c)
			{
			case 'v':// v c'est une info de vertex
				fscanf(fichier,"%c ", &c);
				switch (c)
				{
				case ' ':// vertex normal
					readVertex();
					break;
				case 'n':// normal du vertex
					readNormal();
					useNorm = true;
					break;
				case 't'://tex coord du vertex
					readTexture();
					useTex = true;
					break;
				}
				break;
			case 'f':// f c'est une face
				readFace();
				break;
			}
			// on passe a la ligne suivante.
			fgets(temp, 100, fichier);


		}
		ret = OBJ_READER_FILE_READ_OK;
	}
	fclose(fichier);
	return ret;
}



void OBJ_Reader::readVertex()
{
	OBJ_R_Vertex tempVector;
	// on lit dans le fichier les infos du vertex
	fscanf(fichier,"%f %f %f",&tempVector.x,&tempVector.y,&tempVector.z);
	// et on ajoute le vecteur (au sens vecteur3D) a notre vecteur de stockage (std::vector)
	vertexTab.push_back(tempVector);
}

void OBJ_Reader::readNormal()
{
	OBJ_R_Vertex tempVector;
	// on lit dans le fichier les infos du vertex
	fscanf(fichier," %f %f %f",&tempVector.x,&tempVector.y,&tempVector.z);
	// et on ajoute le vecteur (au sens vecteur3D) a notre vecteur de stockage (std::vector)
	normalTab.push_back(tempVector);
}


void OBJ_Reader::readTexture()
{
	OBJ_R_Texture temp;
	// lecture des info de texture
	fscanf(fichier," %f %f",&temp.u , & temp.v);
	// et sauvegarde de ses infos.
	textureTab.push_back(temp);
}
void OBJ_Reader::readFace()
{
	OBJ_R_Face temp;
	int t[9];
	if (useTex)
	{
		if(useNorm)
		{
			fscanf(fichier," %d/%d/%d %d/%d/%d %d/%d/%d",&t[0],&t[1],&t[2],&t[3],
														&t[4],&t[5],&t[6],&t[7],&t[8]);
		}
		else
		{
			fscanf(fichier," %d/%d %d/%d %d/%d",&t[0],&t[1],&t[3],&t[4],&t[6],&t[7]);
			t[2] = 0;
			t[5] = 0;
			t[9] = 0;
		}
	}
	else
	{
		if(useNorm)
		{
			fscanf(fichier," %d/%d %d/%d %d/%d",&t[0],&t[2],&t[3],&t[5],&t[6],&t[8]);
			t[1] = 0;
			t[4] = 0;
			t[7] = 0;
		}
		else
		{
			fscanf(fichier," %d %d %d",&t[0],&t[3],&t[6]);
			t[1] = 0;
			t[2] = 0;
			t[4] = 0;
			t[5] = 0;
			t[7] = 0;
			t[8] = 0;
		}
	}
	temp.vertex[0] = t[0]-1;
	temp.vertex[1] = t[3]-1;
	temp.vertex[2] = t[6]-1;
	temp.texture[0] = t[1]-1;
	temp.texture[1] = t[4]-1;
	temp.texture[2] = t[7]-1;
	temp.normal[0] = t[2]-1;
	temp.normal[1] = t[5]-1;
	temp.normal[2] = t[8]-1;
	faceTab.push_back(temp);
}

void OBJ_Reader::print()
{
	FILE *err = fopen("./coucou.txt","w");
	int i;
	for (i=0;i<vertexTab.size();i++)
	{
		fprintf(err,"v %f %f %f\n",vertexTab[i].x,vertexTab[i].y,vertexTab[i].z);
	}
	fprintf(err,"#%d verticies\n",vertexTab.size());
	
	for (i=0;i<textureTab.size();i++)
	{
		fprintf(err,"vt %f %f\n",textureTab[i].u,textureTab[i].v);
	}
	fprintf(err,"#%d texture info\n",textureTab.size());
	
	for (i=0;i<normalTab.size();i++)
	{
		fprintf(err,"v %f %f %f\n",normalTab[i].x,normalTab[i].y,normalTab[i].z);
	}
	fprintf(err,"#%d normal info\n",textureTab.size());
	
	for (i=0;i<faceTab.size();i++)
	{
		fprintf(err,"f %d/%d %d/%d %d/%d\n",faceTab[i].vertex[0],faceTab[i].texture[0],faceTab[i].vertex[1],faceTab[i].texture[1],faceTab[i].vertex[2],faceTab[i].texture[2]);
	}
	fprintf(err,"#%d faces\n",faceTab.size());
	fclose(err);
}

void OBJ_Reader::setFile(char *fichier)
{
	file=fichier;
}


std::vector <OBJ_R_Vertex> OBJ_Reader::getVertex()
{
	return vertexTab;
}

std::vector <OBJ_R_Vertex> OBJ_Reader::getNormal()
{
	return normalTab;
}

std::vector <OBJ_R_Texture> OBJ_Reader::getTexture()
{
	return textureTab;
}

std::vector <OBJ_R_Face> OBJ_Reader::getFace()
{
	return faceTab;
}


