// OBJ_Reader.h: interface for the OBJ_Reader class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(READER_H)
#define OBJ_READER_H

#define OBJ_READER_FILE_NOT_EXIST 100;
#define OBJ_READER_FILE_READ_OK 0;


#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// la classe vector de la STL
#include <vector>


	

/**
* la class pour charger un vertex <i>(devrait tre une structure!)</i>
*/
class OBJ_R_Vertex
{
public:
	/**
	* la cooronnes x du vertex
	*/
	float x;
	/**
	* la cooronnes y du vertex
	*/
	float y;
	/**
	* la cooronnes z du vertex
	*/
	float z;
};

/**
* la class pour charger les coordonnes de textures <i>(devrait tre une structure!)</i>
*
* Comment par Hillaire Sbastien
*/
class OBJ_R_Texture
{
public:
	/**
	* La coordonne de texture sur l'axe X
	*/
	float u;
	/**
	* La coordonne de texture sur l'axe Y
	*/
	float v;
};

/**
* la class pour charger les faces du model <i>(devrait tre une structure!)</i>
*
* Comment par Hillaire Sbastien
*/
class OBJ_R_Face
{
public:
	/**
	* Les 3 identifiants des vertex de la face
	*/
	int vertex[3];
	/**
	* Les 3 coordonnes de textures des vertex de la face
	*/
	int texture[3];
	/**
	* Les 3 normals des vertex de la face
	*/
	int normal[3];
};


/**
* la class pour charger un model stock dans le format .obj (wavefront)
*
* Comment par Hillaire Sbastien
*/
class OBJ_Reader  
{
private:
	/**
	* Le fichier de lecture du model
	*/
	FILE *fichier;
	/**
	* Le chemin du fichier du model
	*/
	char *file;

	/**
	* Le vecteur contenant tout les vertex charg
	*/
	std::vector <OBJ_R_Vertex> vertexTab;
	/**
	* Le vecteur contenant tout les normales charges
	*/
	std::vector <OBJ_R_Vertex> normalTab;
	/**
	* Le vecteur contenant tout les coordonnes de textures charges
	*/
	std::vector <OBJ_R_Texture> textureTab;
	/**
	* Le vecteur contenant tout les faces charges
	*/
	std::vector <OBJ_R_Face> faceTab;

	/**
	* Lit un vertex et le stocke dans le vecteur appropri
	*/
	void readVertex();
	/**
	* Lit une normale et la stocke dans le vecteur appropri
	*/
	void readNormal();
	/**
	* Lit une coordonne de texture et la stocke dans le vecteur appropri
	*/
	void readTexture();
	/**
	* Lit une face et la stocke dans le vecteur appropri
	*/
	void readFace();

	/**
	* est ce que le model utilise des coordonne de texture ?
	*/
	bool useTex;
	/**
	* est ce que le model a des normales par vertex ?
	*/
	bool useNorm;

public:

	/**
	* Constructeur
	*
	* @param fileName le chemin du fichier au format wavefront obj  charger.
	*/
	OBJ_Reader(char *fileName);
	/**
	* Destructeur
	*/
	virtual ~OBJ_Reader();

	/**
	* Lit le fichier
	*
	* @return OBJ_READER_FILE_READ_OK si le fichier  t lut,OBJ_READER_FILE_NOT_EXIST sinon.
	*/
	int readFile();

	/**
	* Afiche toutes les informations du model dans la console
	*/
	void print();
	/**
	* Affecte le chemin du fichier  lire
	*
	* @param fileName le chemin du fichier au format wavefront obj  charger.
	*/
	void setFile(char*);

	/**
	* Permet d'obtenir le veteur de vertices.
	*/
	std::vector <OBJ_R_Vertex> getVertex();
	/**
	* Permet d'obtenir le veteur de normal.
	*/
	std::vector <OBJ_R_Vertex> getNormal();
	/**
	* Permet d'obtenir le veteur de coordonnes de texture.
	*/
	std::vector <OBJ_R_Texture> getTexture();
	/**
	* Permet d'obtenir le veteur de face.
	*/
	std::vector <OBJ_R_Face> getFace();

};

#endif // !defined(AFX_OBJ_READER_H__446292F3_C835_48BB_AE24_7E073EF4B3CC__INCLUDED_)
