// Objet.h: interface for the Objet class.
//
//////////////////////////////////////////////////////////////////////

#ifndef OBJET_H
#define OBJET_H

#include "RULES_Vector.h"
#include "Model.h"
#include "ModelDrawer.h"

/**
* La classe permettant de grer les objet de la scne :
*	- la position
*	- le model
*	- le rendu du model
*/
class Objet  
{
public:

	/**
	* Accd  la position de l'objet
	*
	* @return la position
	*/
	RULES_Vector getPosition();

	/**
	* Affect la position de l'objet
	*
	* @param pos la nouvelle position
	*/
	void setPosition(RULES_Vector pos);


	/**
	* Affecte le renderer de cette objet pour qu'il puisse ce dessiner.
	*(c'est une mthode comme une autre...)
	*
	* @param drawer le renderer de cette objet
	*/
	void setModelDrawer(ModelDrawer * drawer);

	/**
	* Accder au renderer affichant cette objet.
	*
	* @return le pointeur sur le renderer de cette objet
	*/
	ModelDrawer * getModelDrawer();
	

	/**
	* Accder au model de cette objet
	*
	* @return le pointeur sur le model de l'objet
	*/
	Model * getModel();

	/**
	* Affecte le model de cette objet
	*
	* @param model le model sur lequel va pointer l'objet(pas de copie)
	*/
	void setModel(Model * model);


	/**
	* L'objet est-il static(comme l'immeuble) ou dynamique(comme la loutre)
	*
	* @return true si l'objet est static; false sinon.
	*/
	bool isStatic();

	/**
	* Affecte le statisme/dynamisme de l'objet.
	*
	* @param isStatic true si l'objet est static; false sinon.
	*/
	void setIsStatic(bool isStatic);


	/**
	* Appeler cette mthode pour afficher le model dans la scne.
	* Mthode  implmenter.
	*/
	virtual void draw();

	/**
	* Constructeur
	*/
	Objet();

	/**
	* Destructeur
	*/
	virtual ~Objet();

protected:

	/**
	* Le renderer de cette objet
	*/
	ModelDrawer * modelDrawer;

	/**
	* Indique si l'objet est statique ou non
	*/
	bool statique;

	/**
	* Le model de l'objet
	*/
	Model * model;

	/**
	* La position de l'objet
	*/
	RULES_Vector position;
};

#endif 
