// ParticleEngine.cpp: implementation of the ParticleEngine class.
//
//////////////////////////////////////////////////////////////////////

#include "ParticleEngine.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ParticleEngine::ParticleEngine()
{
	//allocation et init des particule
	particles = new Particle[200];
	for (int i = 0; i < 200;i++)
	{
		particles[i].position.setCoord(0,0,0);
		particles[i].direction.setCoord(0,10,0);
		particles[i].vie = 0;
		particles[i].init = false;
		particles[i].rotation = 0;
	}
	previousTime = glutGet(GLUT_ELAPSED_TIME);
	beginTime = glutGet(GLUT_ELAPSED_TIME);
	delay = 50;
	particleNumber = 200;
	currentIndice = 0;
	//chargement de la texture des particules
	image = new RULES_Texture();
	image->loadBMP("./ressources/textures/particle.bmp",0,GL_LINEAR_MIPMAP_LINEAR,GL_LINEAR,GL_CLAMP,GL_CLAMP,0,false,0,0,255,0,1);
}

ParticleEngine::~ParticleEngine()
{
	//relache les ressource si celle ci on t allouer
	if (particles != NULL)
	{
		delete [] particles;
		particles = NULL;
	}
	if (image != NULL)
	{
		delete image;
		image = NULL;
	}
}

void ParticleEngine::setParticlesNumber( int number)
{
	//change le nombre de particule dans que le moteur  a grer
	delete [] particles;
	particles = new Particle[number];;
	for (int i = 0; i < number;i++)
	{
		//initialisation par dfaut
		particles[i].position.setCoord(0,0,0);
		particles[i].direction.setCoord(0,10,0);
		particles[i].vie = 0;
		particles[i].init = false;
		particles[i].rotation = 0;
	}
	particleNumber = number;
}

void ParticleEngine::setPreviousPosition(RULES_Vector * pos)
{
	this->previousPosition.setCoord(pos->getX(),pos->getY(),pos->getZ());
}

void ParticleEngine::frame(RULES_Vector * pos)
{
	float factor = (glutGet(GLUT_ELAPSED_TIME) - this->previousTime)*0.01;
	this->previousTime = glutGet(GLUT_ELAPSED_TIME);
	// on genere une nouvelle particule
	if (this->beginTime + this->delay < glutGet(GLUT_ELAPSED_TIME))
	{
		this->beginTime = glutGet(GLUT_ELAPSED_TIME);
		particles[currentIndice].position.setCoord(pos->getX(),pos->getY(),pos->getZ());
		if (!((pos->getX() == previousPosition.getX())&&(pos->getY() == previousPosition.getY())))
		{
			particles[currentIndice].direction = previousPosition - *pos;
			particles[currentIndice].direction *=-1 ;
		}
		else
		{
			particles[currentIndice].direction.setCoord(10,0,0);
			particles[currentIndice].direction.rotate(rand(),0,0,1);
			particles[currentIndice].direction * cos((float)rand());
		}
		particles[currentIndice].vie = 0;
		particles[currentIndice].color[0] = fabs(cos((float)rand()));
		particles[currentIndice].color[1] = fabs(cos((float)rand()));
		particles[currentIndice].color[2] = fabs(cos((float)rand()));
		particles[currentIndice].init = true;
		particles[currentIndice].rotation = rand();
		currentIndice ++;
		if (currentIndice >= particleNumber)
		{
			currentIndice = 0;
		}
	}

	/**** Affichage des particules ****/

	// on desactive le depth test
	glDisable(GL_DEPTH_TEST);
	// on passe en mode projection
	glMatrixMode(GL_PROJECTION);
		
				// on sauvegarde la matrice de projection
	glPushMatrix();
	
		// on nettoie la matrice de projection
		glLoadIdentity();
		
		// on passe en mode ortho de la taille de l'ecran
		glOrtho(0,1024,0,768,-1,1);	
		
		// on passe en mode modelview
		glMatrixMode(GL_MODELVIEW);
		
		// on sauvegarde la matrice de modelview
		glPushMatrix();
		
			// on nettoie la matrice de projection
			glLoadIdentity();

			glEnable(GL_TEXTURE_2D);
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA,GL_ONE);
			glBindTexture(GL_TEXTURE_2D, this->image->getId());
			glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
			glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
			// on fait progresser toutes les particules
			for (int i = 0 ; i < this->particleNumber; i++)
			{
				if (particles[i].init)
				{
					glMatrixMode(GL_TEXTURE);
					glLoadIdentity();
					glTranslatef(0.5,0.5,0);
					glRotatef(particles[i].rotation,0,0,1);
					glTranslatef(-0.5,-0.5,0);
					glMatrixMode(GL_MODELVIEW);
					glBegin(GL_QUADS);
						// on l'affiche
						float scale = 1-(float)particles[i].vie/50;
						glColor4f(particles[i].color[0],particles[i].color[1],particles[i].color[2],scale);
						glTexCoord2f(0,0);
						glVertex2f(particles[i].position.getX() - 16,particles[i].position.getY() - 16);
						glTexCoord2f(1,0);
						glVertex2f(particles[i].position.getX() + 16,particles[i].position.getY() - 16);
						glTexCoord2f(1,1);
						glVertex2f(particles[i].position.getX() + 16,particles[i].position.getY() + 16);
						glTexCoord2f(0,1);
						glVertex2f(particles[i].position.getX() - 16,particles[i].position.getY() + 16);
					glEnd();

					particles[i].position = particles[i].position+(particles[i].direction*factor);
					particles[i].direction.setOneCoord(1,particles[i].direction.getY() - 0.981*factor);
					particles[i].vie += factor;
					particles[i].rotation += factor*2;
					if (particles[i].vie > 50)
					{
						particles[i].init = false;
					}
				}
			}
			glDisable(GL_BLEND);
			glMatrixMode(GL_PROJECTION);
		
		// et on recupere la matrice de projection sauvegard precedement
		glPopMatrix();	
		
		// on selectionne la matrice de projection modelview
		glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
	// on desactive le depth test
	glEnable(GL_DEPTH_TEST);
	glMatrixMode(GL_TEXTURE);
	glLoadIdentity();
	glMatrixMode(GL_MODELVIEW);

	this->setPreviousPosition(pos);
	glColor4f(1,1,1,1);
}
