// RULES_Textures.cpp: implementation of the RULES_Textures class.
//
//////////////////////////////////////////////////////////////////////

#include "RULES_Texture.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

RULES_Texture::RULES_Texture()
{
	id=0;
	//genere l'objet de texture OpenGL en mmoire vido
	//glGenTextures(1,&id);
}

RULES_Texture::~RULES_Texture()
{
	//supprime l'objet de texture OpenGL en mmoire vido
	glDeleteTextures(1,&id);
}

//_ retourne l'identifiant OpenGL de la texture.
unsigned int RULES_Texture::getId()
{
	return id;
}

// _ retourne true si le fichier a t trouv, sinon false.
// PARAMETRE :
// _ filePath : chemin du fichier.
// _ useMipMap : spcifie si l'on doit constuire l'objet de texture avec ces mipmap.
// _ magFilter : identifiant OpenGL spcifiant le filtrage si la texture est agrandit sur l'cran.
// _ minFilter : identifiant OpenGL spcifiant le filtrage si la texture est rduite sur l'cran.
//=> ATTENTION : si vous utilisez le mipmap alors utilis un filtrage OpenGL valide pour cela.
// _ sWrap : mode de rpetition de texture OpenGL sur la composante u de la texture
// _ tWrap : mode de rpetition de texture OpenGL sur la composante v de la texture
// _ levelOfDetail : spcifie le niveau de dtail de la texture en meilleur qualit. 0->qualit maximum, 1-> lod de la texture est divis par 2,etc...
// _ generateAlpha : spcifie si l'on doit gnerer une texture avec un canal alpha gnerer avec nue couleur de transparence.
// _ rBlended : composante rouge de la couleur  associ a un alpha prit.
// _ gBlended :  composante verte de la couleur  associ a un alpha prit.
// _ bBlended :  composante bleue de la couleur  associ a un alpha prit.
// _ alphaBlendValue : la valeur alpha plac la ou les composantes 'r''g''b'Blend ont t trouves.
// _ alphaDefault : valeur alpha  placer si la couleur de Blend n'a pas t trouv.
bool RULES_Texture::loadBMP(char *filePath, bool useMipMap, int magFilter, int minFilter, int sWrap, int tWrap, unsigned int levelOfDetail, bool generateAlpha, unsigned char rBlended, unsigned char gBlended, unsigned char bBlended, unsigned char alphaBlendValue, unsigned char alphaDefault)
{
	bool retFileExist=false;
	FILE *f;
	bool NeedBlend=false;			//permet de vrifier si on doit utiliser la transparence

	f=fopen(filePath,"r");
	if(f!=NULL)
	{
		retFileExist=true;
		if(id!=0)	//si on recharge la texture, il faut effacer l'ancienne
		{
			glDeleteTextures(1,&id);
		}
		glGenTextures(1,&id);

		////////////////////////////CHARGEMENT DE LA TEXTURE APROPRIE/////////////////////////
		
		AUX_RGBImageRec * textureload;				//chargeur de bmp aux
		textureload=auxDIBImageLoad(filePath);		//chargement de la texture dans objet aux
		
		if(levelOfDetail<0)
		{//vrification validit de detailDown
			levelOfDetail=0;
		}
		int detailSizeFactor=pow(2,levelOfDetail);								//dcalage a effectuer entre les pixel pour baisser les dtails
		int size=(textureload->sizeX*textureload->sizeY*4)/detailSizeFactor;//taille du buffer image avec baisse de dtail prise en compte
		if(size<4)
		{//vrification validit de la taille minimum du buffer image ( 4 -> r,g,b,a )
			size=4;
		}
		unsigned char *temp=new unsigned char[ size ];	//cration buffer de texture avec les composantes alpha
		unsigned char * pc=textureload->data;			//initialisation pointeur temporaire sur le buffer image
		unsigned char r,v,b;
		int tailleDestX=textureload->sizeX/detailSizeFactor;
		int tailleDestY=textureload->sizeY/detailSizeFactor;
		if(tailleDestX<1)
		{//vrification de la bornes x de la texture destination
			tailleDestX=1;
		}
		if(tailleDestY<1)
		{//vrification de la bornes y de la texture destination
			tailleDestY=1;
		}
		int nbSourceTexel=textureload->sizeX*textureload->sizeY;//nb de texels de la texture  charger
		int nbDestTexl=size/4;									//nb de texels de la texture  envoyer
		
		//la texture source contient 3 composant (rgb)
		//la texture destination contient 4 composant (rgba)
		
		for(int i=0;i<tailleDestX;i++)
		{
			for(int j=0;j<tailleDestY;j++)
			{
				int RX=(float)textureload->sizeX*((float)i/(float)tailleDestX);//coordonne x du pixel a choisir dans la texture source
				int RY=(float)textureload->sizeY*((float)j/(float)tailleDestY);//coordonne y du pixel a choisir dans la texture source
				
				//extraction et affectation de la composante r
				r=pc[RX*3+RY*3*textureload->sizeX];
				temp[i*4+j*4*tailleDestX]=r;
				
				//extraction et affectation de la composante g
				v=pc[RX*3+RY*3*textureload->sizeX+1];
				temp[i*4+j*4*tailleDestX+1]=v;
				
				//extraction et affectation de la composante b
				b=pc[RX*3+RY*3*textureload->sizeX+2];
				temp[i*4+j*4*tailleDestX+2]=b;

				if( r==rBlended && v==gBlended && b==bBlended )
				{//la couleur correspond a la couleur de blend dsirer
					temp[i*4+j*4*tailleDestX+3]=alphaBlendValue;				//on assigne le alpha de transparence
					NeedBlend=true;
				}
				else
				{
					temp[i*4+j*4*tailleDestX+3]=alphaDefault;
				}
			}
		}

		//initialisation de l'objet de texture
		glBindTexture(GL_TEXTURE_2D,id);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,sWrap);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,tWrap);

		//filtrage a appliquer en cas d'tirement et de ressrage de la texture
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,magFilter);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,minFilter);
		
		if(useMipMap)
		{//on doit utiliser les mipmap
			if( NeedBlend && generateAlpha)
			{//on doit gnerer une composante alpha dans la texture
				gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA,tailleDestX,tailleDestY,GL_RGBA,GL_UNSIGNED_BYTE,temp);
			}
			else
			{//on ne doit pas gnerer une composante alpha dans la texture
				gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB,tailleDestX,tailleDestY,GL_RGBA,GL_UNSIGNED_BYTE,temp);
			}
		}
		else
		{//on ne doit pas utiliser les mipmap
			if( NeedBlend && generateAlpha)
			{//on doit gnerer une composante alpha dans la texture
				glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,tailleDestX,tailleDestY,0,GL_RGBA,GL_UNSIGNED_BYTE,temp);
			}
			else
			{//on ne doit pas gnerer une composante alpha dans la texture
				glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,tailleDestX,tailleDestY,0,GL_RGBA,GL_UNSIGNED_BYTE,temp);
			}
		}
		// added by bafman
		delete temp;
	}

	return retFileExist;
}
