// RULES_Textures.h: interface for the RULES_Textures class.
//
//////////////////////////////////////////////////////////////////////

#ifndef RULES_TEXTURE
#define RULES_TEXTURE

//bibliothque C&C++
#include <stdio.h>
#include "math.h"

//bibliothque OpenGL
#ifdef WIN32
#include <windows.h>
#endif
#include <GL\gl.h>
#include "GL\glut.h"
#include "GL\glaux.h"

/**
* La classe de chargement de texture  l'aide de glaux.
*/
class RULES_Texture  
{
private:

	/**
	* L'identifiant OpenGL de la texture
	*/
	unsigned int id;

public:
	/**
	* Charge une texture bmp du HDD
	*
	* @param *filePath le chemin de l'image bmp a lire.
	* @param useMipMap indique si on doit gnerer des mipmap pour l'objet de texture
	* @param magFilter indique le filtrage dans le cas de magnification de la texture
	* @param minFilter indique le filtrage dans le cas de minification de la texture
	* @param sWrap indique le type de rpetition de texture pour la coordonne s
	* @param tWrap indique le type de rpetition de texture pour la coordonne t
	* @param levelOfDetail le nombre de fois a faire baiss la qualit.de la texture.(0=qualit de l'authentique)
	* @param generateAlpha indique si l'on doit gnerer un canal alpha pour la texture
	* @param rBlended indique la composante rouge de la couleur a laquel on affecte la valeur alphaBlendValue
	* @param gBlended indique la composante verte de la couleur a laquel on affecte la valeur alphaBlendValue
	* @param bBlended indique la composante bleue de la couleur a laquel on affecte la valeur alphaBlendValue
	* @param alphaBlendValue indique la valeur de la composant alpha dans le cas de la couleur de transparence 
	* @param alphaDefault indique la valeur de la composant alpha dans le cas d'une couleur normal
	*
	* @return retourn true si le fichier a bien t charger
	*/
	bool loadBMP(char *filePath, bool useMipMap, int magFilter, int minFilter, int sWrap, int tWrap, unsigned int levelOfDetail, bool generateAlpha, unsigned char rBlended, unsigned char gBlended, unsigned char bBlended, unsigned char alphaBlendValue, unsigned char alphaDefault);

	/**
	* retourne l'identifiant OpenGL de la texture
	*
	* @return l'identifiant
	*/
	unsigned int getId();

	/**
	* Constructeur
	*/
	RULES_Texture();
	/**
	* Destructeur
	*/
	~RULES_Texture();

};

#endif
