// SoundManager.cpp: implementation of the SoundManager class.
//
//////////////////////////////////////////////////////////////////////

#include "SoundManager.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SoundManager::SoundManager()
{
	// par defaut on utilise :
	// les effets
	useEffect = true;
	// pas de musique
	useMusic = 0;
	// niveau sonore des effet a 100 (sur 255)
	effectSoundLevel = 100;
	// niveau sonor de la musique a 100 (sur 255)
	musicSoundLevel = 100;
	// on cre une nouvelle playList
	playList = new PlayList();
	
	// initialisation de fmod
	// param :	frequence,
	//			nombre max de channels, 
	//			init_flag => defaut = 0
	if (!FSOUND_Init(44100, 32, 0))
	{
		// FMOD pas initialis
		// on utilise pas les effets
		useEffect = false;
		// et pas la musique non plus
		useMusic = 0;
	}
	else
	{
		// FMOD initialis
    
		// on charge le sample en memoire
		// param :	choisi un chanel libre,
		//			nom du fichier  charger
		//			mode de chargement
		//			taille du buffer de son => 0 si c'est un fichier
		detonation	= FSOUND_Sample_Load(FSOUND_FREE, "./ressources/sons/detonation.ogg", FSOUND_LOOP_OFF | FSOUND_NORMAL | FSOUND_HW2D, 0);
		loutre[0]	= FSOUND_Sample_Load(FSOUND_FREE, "./ressources/sons/loutre1.wav", FSOUND_LOOP_OFF | FSOUND_NORMAL | FSOUND_HW2D, 0);
		loutre[1]	= FSOUND_Sample_Load(FSOUND_FREE, "./ressources/sons/loutre2.wav", FSOUND_LOOP_OFF | FSOUND_NORMAL | FSOUND_HW2D, 0);
		loutre[2]	= FSOUND_Sample_Load(FSOUND_FREE, "./ressources/sons/loutre3.wav", FSOUND_LOOP_OFF | FSOUND_NORMAL | FSOUND_HW2D, 0);
		clic		= FSOUND_Sample_Load(FSOUND_FREE, "./ressources/sons/clic.ogg", FSOUND_LOOP_OFF | FSOUND_NORMAL | FSOUND_HW2D, 0);
		boom		= FSOUND_Sample_Load(FSOUND_FREE, "./ressources/sons/boom.ogg", FSOUND_LOOP_OFF | FSOUND_NORMAL | FSOUND_HW2D, 0);
		
		// specifie le taille du buffer a utiliser pour le prochain
		// stream charg (en millisecondes)
		FSOUND_Stream_SetBufferSize(2000);

		// lecture du repertoire de base pour la playlist
		playList->readBaseRep("./ressources/playlist/base_repertoire.txt");
		// lecture des informtion de la playlist
		playList->readPlayList("./ressources/playlist/playlist.m3u");
		if (playList->isOk())
		{
			// la playlist est valide
			// on utilise la playlist pour la musique
			useMusic = 0;
			// par defaut en mode random
			playList->setRandom(true);
			// on joue la prochaine musique (qui est aussi la premiere)
			playStream();


		}
		else
		{
			// la  playlist n'est pas valide
			// on utilise le MIDI
			useMusic = 0;
			// on charge le fichier MIDI
			musique = FMUSIC_LoadSong("./ressources/sons/musique.mid");
			// on joue directement la musique
			FMUSIC_PlaySong(musique);
			// en specifiant le volume
			FMUSIC_SetMasterVolume(musique,musicSoundLevel);
			// et on met en pause le MIDI
			FMUSIC_SetPaused(musique, true);
		}
	
	}
}

SoundManager::~SoundManager()
{
    delete playList;
	// on libere lee sample
	FSOUND_Sample_Free(detonation);
	FSOUND_Sample_Free(loutre[0]);
	FSOUND_Sample_Free(loutre[1]);
	FSOUND_Sample_Free(loutre[2]);
	FSOUND_Sample_Free(clic);
	FSOUND_Sample_Free(boom);
	// on libere le MIDI
    FMUSIC_FreeSong(musique);

	// et on ferme fmod
    FSOUND_Close();

}

void SoundManager::playDetonation()
{
	if (useEffect)
	{
		// on utilise les effets donc on peut jouer le son
		int tmp = FSOUND_PlaySound(FSOUND_FREE, detonation);
		// on specifie le volume
		FSOUND_SetVolume(tmp,effectSoundLevel);
	}
}

void SoundManager::playLoutre()
{
	if (useEffect)
	{
		// on utilise les effets donc on peut jouer le son
		// on choisit un son de loutre au hazard parmis 3
		int i = (int)(fabs(cos((float)rand()))*3);
		// on le joue
		int tmp = FSOUND_PlaySound(FSOUND_FREE, loutre[i]);
		// et on specifie le volume
		FSOUND_SetVolume(tmp,effectSoundLevel);
	}
}

void SoundManager::playClic()
{
	if (useEffect)
	{
		// on utilise les effets
		// donc on joue le son
		int tmp = FSOUND_PlaySound(FSOUND_FREE, clic);
		// on specifie le volume
		FSOUND_SetVolume(tmp,effectSoundLevel);
	}	
}

void SoundManager::checkSound()
{
	if (this->playList->isOk())
	{
		// la playlist est valide donc c'est qu'on joue des MP3
		if (FSOUND_Stream_GetTime(stream) > FSOUND_Stream_GetLengthMs(stream) -2000)
		{
			// la musique courante est a moins de 2 seconde de la fin
			// on joue la suivante
			SoundManager::getInstance()->playStream();
		}
	}

	
}

void SoundManager::playBoom()
{
	if (useEffect)
	{
		// on utilise les effets
		// donc on joue le son
		int tmp = FSOUND_PlaySound(FSOUND_FREE, boom);
		// et on specifie le volume (legerement au dessus de la normal pour etre sure qu'il depasse les autres...)
		// ca ne pose pas de probleme avec FMOD, car si le son depasse 255, il est remapp...
		FSOUND_SetVolume(tmp,effectSoundLevel+10);
	}
}

void SoundManager::setUseEffect(bool use)
{
	this->useEffect = use;
}

void SoundManager::setUseMusic(bool use)
{
	switch (useMusic)
	{
	case 0:
		// on ne joue pas de musique pour l'instant
		if (use)
		{
			// on decide d'en jouer...
			if (playList->isOk())
			{
				// la playlit est valide, on va donc jouer des MP3...
				// on passe en mode playlist
				useMusic = 2;
				// et on joue la prochaine musique
				this->playStream();
			}
			else
			{
				// la playlist n'est pas valide, on joue donc le MIDI
				// on passe en mode MIDI
				useMusic = 1;
				// et on enleve la pause sur le MIDI
				FMUSIC_SetPaused(musique, false);   
			}
		}
		break;
	case 1:
		// on joue le MIDI
		// on le met en pause
		if (FMUSIC_GetPaused(musique) == false)
		{
			FMUSIC_SetPaused(musique, true);   
		}
		// et on passe en mode sans musique, la prochaine fois que sera appel cette methode, elle
		// relancera le MIDI dans le "case 0:"
		useMusic = 0;
		break;
	case 2:
		// on joue la playlist
		// on ferme le stream
		FSOUND_Stream_Close(stream);
		// et on passe en mode sans musique. le stream sera relanc si on rappel cette methode.
		useMusic = 0;
		break;
	}

}

void F_CALLBACKAPI SoundManager::playStream()
{
	if ((playList->isOk())&&(this->useMusic == 2))
	{
		// la playlist est valide ET on veut jouer des MP3...
		// on recupere le chemin du prochain MP3
		char * mp3 = playList->getNextMp3();
		
		
		if (strlen(mp3)>0)
		{
			// le MP3 retourn est valide.
			// on charge le stream en memoire
			stream = FSOUND_Stream_OpenFile( mp3,FSOUND_NORMAL| FSOUND_MPEGACCURATE,0);
		}
		// on delete le chemin du MP3 (voir la doc de playList::getNextMp3());
		delete [] mp3;
    
		// et on le joue
		streamChannel = FSOUND_Stream_Play(FSOUND_FREE, stream);

		// on change le volume
		FSOUND_SetVolume(streamChannel,musicSoundLevel);
	}

}

signed char F_CALLBACKAPI callback(FSOUND_STREAM *stream, void *buff, int len, int param)
{
	// methode deprecated...
	if (!buff)
	{
		SoundManager::getInstance()->playStream();
	}
	return true;
}

void SoundManager::setEffectSoundLevel(int level)
{
	// on test si le son est dans la bonne plage de valeurs
	// si non, on le remappe dans la plage
	if (level > 255)
	{
		level = 255;
	}
	if (level < 0)
	{
		level = 0;
	}
	// on change le volume qui sera appliqu lors du prochain appel au methode pour
	// jouer des sons
	this->effectSoundLevel = level;
}

void SoundManager::setMusicSoundLevel(int level)
{
	// on test si le son est dans la bonne plage de valeurs
	// si non, on le remappe dans la plage
	if (level > 255)
	{
		level = 255;
	}
	if (level < 0)
	{
		level = 0;
	}
	// on change le volume
	this->musicSoundLevel = level;
	// et on l'applique au MIDI et au stream qui sont pottentiellement en train de jouer...
	FMUSIC_SetMasterVolume(musique,musicSoundLevel);
	FSOUND_SetVolume(streamChannel,musicSoundLevel);
}

int SoundManager::getEffectSoundLevel()
{
	return this->effectSoundLevel;
}

int SoundManager::getMusicSoundLevel()
{
	return this->musicSoundLevel;
}

PlayList * SoundManager::getPlayList()
{
	return this->playList;
}

bool SoundManager::isUsingEffect()
{
	return this->useEffect;
}

int SoundManager::musicUsingMode()
{
	return this->useMusic;
}
