// SoundManager.h: interface for the SoundManager class.
//
//////////////////////////////////////////////////////////////////////

/**
 * documentation ralis par Michel de VERDELHAN : michel.deverdelhan@free.fr
 * le 20/10/2004
 */

#ifndef SOUNDMANAGER_H
#define SOUNDMANAGER_H

#include "fmod.h"//ne pas oublier de linker fmodvc.lib
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "PlayList.h"

// definition de la callback de fmod
/**
 * cette fonction etait sense etre utilise comme callback pour specifi la fin des fichiers de son
 * mai il s'est avr que ce n'etait pas utilisable, vu que les point d'appels devait etres specifie
 * par le soundeur, ce qui empechait une utilisation (et un changement) facile des sons, on a du changer
 * la technique utilise en verifiant a chaque tour de la boucle principale si la musique est termine.
 * C'est de l'attente active, c'est nul mais on a pas prit le temps de trouver mieux...
 */
signed char F_CALLBACKAPI callback(FSOUND_STREAM *stream, void *buff, int len, int param);

/**
 * Classe gerant le system de son du jeu.
 * </BR><B>Cette classe utilise le pattern <I>SINGLETON</I></B>
 * </BR> la classe charge les sons utilis dans le jeu (detonation de l'arme, bruit debiles des loutres,
 * clic de la souris dans l'interface, explosion dans l'immeuble) et la musique.
 * </BR> pour la musique, la classe peut soit utiliser une playListe winamp soit un midi.
 */
class SoundManager  
{
private:
	/**
	 * le channel de la stream FMOD utilis pour jouer la musique lors de l'utilisation d'une playListe.
	 */
	int streamChannel;

	/**
	 * le niveau des effetes sonores dans le jeu.
	 */
	int effectSoundLevel;

	/**
	 * le niveau sonore de la musique (playListe ou MIDI) dans le jeu
	 */
	int musicSoundLevel;

	/**
	 * definit si on doit utiliser les effets sonors dans le jeu
	 */
	bool useEffect;

	/**
	 * definit le mode de musique utilise. Le mode de musique est definit a l'instanciation de la classe, si il n'y a pas de
	 * playliste valide, la methode sera MIDI, sinon, si il y a une playlist, elle sera utilise.
	 * </BR> les modes des musique sont :
	 * <LI> 0 => pas de musique
	 * <LI> 1 => MIDI
	 * <LI> 2 => playlist
	 */
	int useMusic;

	/**
	 * verifie l'existense d'une playList valide dans le repertoire "ressources/playlist"
	 */
	bool existPlayList;
	
	/**
	 * le sample FMOD du bruit de la detonation
	 */
	FSOUND_SAMPLE *detonation;

	/**
	 * le tableau de sample FMOD des bruits des loutres
	 */
	FSOUND_SAMPLE *loutre[3];
	
	/**
	 * le sample FMOD du bruit du clic de la souris dans le menu
	 */
	FSOUND_SAMPLE *clic;

	/**
	 * le sample FMOD du bruit de l'explosion d'une loutre contre l'immeuble
	 */
	FSOUND_SAMPLE *boom;
	
	/**
	 * le module FMOD la musique lors de la lecture de MIDI
	 */
	FMUSIC_MODULE *musique;

	/**
	 * la playliste de mp3
	 */
	PlayList * playList;
	
	/**
	 * la stream FMOD du mp3 jou lors de la lecture de playlist
	 */
	FSOUND_STREAM *stream;

	/**
	 * constructeur par defaut, initialise FMOD, la playList...
	 * </BR> le constructeur est mis en private pour le pattern singleton
	 */
	SoundManager();
public:

	/**
	 * methode retournant le type de musique utilise
	 * @return int:
 	 * <LI> 0 => pas de musique
	 * <LI> 1 => MIDI
	 * <LI> 2 => playlist
	 */
	int musicUsingMode();

	

	void setMusicUsingMode(int musicMode)
	{
		this->useMusic = musicMode;
	}

	/**
	 * methode permettant de savoir si le soundManager utilise les effets sonors
	 * @return true: le soundManager utilise les effets
	 */
	bool isUsingEffect();

	/**
	 * methode retournant la playList (cette methode est utilise dans le menu son pour savoir si la playList est valide)
	 * @return la playList utilise par le soundManager
	 */
	PlayList * getPlayList();

	/**
	 * methode retournant le niveau sonore de la musique
	 * @return int: le niveau sonore de la musique, comprit dans [0..255]
	 */
	int getMusicSoundLevel();

	/**
	 * methode retournant le niveau sonore des effets
	 * @return int: le niveau sonore des effets, comprit dans [0..255]
	 */
	int getEffectSoundLevel();

	/**
	 * methode specifiant le niveau sonore de la musique
	 * @param level: niveau sonore, doit etre comprit entre 0 et 255. Si le niveau sonor n'est pas comprit dans cet interval,
	 * la methode le met dedans.
	 */
	void setMusicSoundLevel(int level);

	/**
	 * methode specifiant le niveau sonor des effets
	 * @param level: niveau sonor des effets, doit etre comprit entre 0 et 255. Si le niveau sonor n'est pas comprit
	 * dans cet interval, la methode le met dedans.
	 */
	void setEffectSoundLevel(int level);

	/**
	 * methode jouant le prochain MP3 de la playlist. Cette methode ne fait rien si le mode de musique n'est pas playlist
	 */
	void F_CALLBACKAPI playStream();

	/**
	 * methode definissant si le soundManager utilise la musique.
	 </BR> cette methode est sans effets si le mode de musique est 0.
	 * @param use: true => uytilisation de la musique.
	 */
	void setUseMusic(bool use);

	/**
	 * methode definissant si le soundManager utilise les effets sonors.
	 * @param use: trus => utilisation des effets.
	 */
	void setUseEffect(bool use);

	/**
	 * methode qui demarre la lecture du son de collision d'une loutre contre l'immeuble
	 */
	void playBoom();

	/**
	 * methode qui demarre la lecture du son de detonation lors d'un tir
	 */
	void playDetonation();

	/**
	 * methode qui demarre la lecture du son de loutre aleatoirement choisit parmit les 3 possible
	 */
	void playLoutre();

	/**
	 * methode qui demarre la lecture du son lors d'un clic souris dans l'interface
	 */
	void playClic();

	/**
	 * methode appel a CHAQUE tour de la boucle principale du jeu, qui verifie si la musique est termine
	 * et qui lance la suivante quand on approche de la fin de celle joue.
	 * </BR> cette methode est de l'attente active, mais c'est la premiere solution trouve au probleme pos par les callback 
	 * FMOD.
	 */
	void checkSound();

	/**
	 * destructeur par defut, ferme les son, libere l'espace allou par FMOD pour la musique, et ferme FMOD
	 */
	virtual ~SoundManager();

	/**
	 * methode retournant l'instance unique de soundManager due a l'utilisation du pattern <B>SINGLETON</B>
	 */
	inline static SoundManager * getInstance(){static SoundManager instance;return &instance;};
};

#endif 
