// collisionManager.h: interface for the CollisionManager class.
//
//////////////////////////////////////////////////////////////////////

#ifndef COLLISIONMANAGER_H
#define COLLISIONMANAGER_H

#include "Loutre.h"
#include "Joueur.h"
#include "Immeuble.h"
#include "RULES_Vector.h"

#define JOUEUR_RADIUS		30

/**
*	Classe permettant de grer les collisions des diverses entites du jeu :
* collisions joueur-environnement,loutre-joueur, loutre-immeuble et loutre-loutre.
*/
class CollisionManager  
{
private:
	/**
	* le nombre de loutre  controller
	*/
	int loutreNumber;
	/**
	* le nombre d'immeuble  controller
	*/
	int immeubleNumber;
	/**
	* le nombre de joueur  controller
	*/
	int joueurNumber;

	/**
	* le pointeur sur le tableau des loutres
	*/
	Loutre * loutres;
	/**
	* le pointeur sur le tableau d'immeubles
	*/
	Immeuble * immeubles;
	/**
	* le pointeur sur le tableau des joueurs
	*/
	Joueur * joueurs;

public:

	/**
	* Affecte le nombre de joueur
	*
	* @param le nombre de joueur
	*/
	void setJoueurNumber(int nbj);
	/**
	* Affecte le nombre de loutres
	*
	* @param le nombre de loutres
	*/
	void setLoutreNumber(int nbl);
	/**
	* Affecte le nombre d'immeuble
	*
	* @param le nombre de immeuble
	*/
	void setImmeubleNumber(int nbi);

	/**
	* Affecte le pointeur sur le tableau de Joueur
	*
	* @param i le pointeur sur le tableau de Joueur
	*/
	void setJoueurs(Joueur* j);
	/**
	* Affecte le pointeur sur le tableau de Loutre
	*
	* @param i le pointeur sur le tableau de Loutre
	*/
	void setLoutres(Loutre* l);
	/**
	* Affecte le pointeur sur le tableau d'Immeuble
	*
	* @param i le pointeur sur le tableau d'Immeuble
	*/
	void setImmeubles(Immeuble* i);

	/**
	* Calcul les collision cette frame
	*/
	void manageCollision();

	/**
	* Constructeur
	*
	* @param lo le pointeur sur le tableau de Loutre
	* @param im le pointeur sur le tableau d'Immeuble
	* @param joueurs le pointeur sur le tableau de Joueur
	* @param nblo le nombre de loutres
	* @param nbim le nombre d'immeuble
	* @param nbjo le nombre de joueur
	*/
	CollisionManager(Loutre *lo, Immeuble *im, Joueur *joueurs, int nblo, int nbim, int nbjo);
	/**
	* Destructeur
	*/
	virtual ~CollisionManager();

};

#endif
